import bpy
from .utils import get_next_camera_name, get_sorted_camera_objects

class CAMERA2VIEW_OT_CreateCamera(bpy.types.Operator):
    bl_idname = "camera2view.create_camera"
    bl_label = "Camera2View"
    bl_description = "Create a new camera from the current 3D view"

    def execute(self, context):
        scene = context.scene
        area = next((a for a in context.window.screen.areas if a.type == 'VIEW_3D'), None)
        if not area:
            self.report({'WARNING'}, "No 3D Viewport found")
            return {'CANCELLED'}

        region = next((r for r in area.regions if r.type == 'WINDOW'), None)
        space = next((s for s in area.spaces if s.type == 'VIEW_3D'), None)
        if not (region and space):
            self.report({'WARNING'}, "Could not find region or space data")
            return {'CANCELLED'}

        view_matrix = space.region_3d.view_matrix
        cam_data = bpy.data.cameras.new(get_next_camera_name())
        cam_obj = bpy.data.objects.new(cam_data.name, cam_data)
        scene.collection.objects.link(cam_obj)

        cam_obj.matrix_world = view_matrix.inverted()
        cam_data.lens = space.lens
        cam_data.clip_start = space.clip_start
        cam_data.clip_end = space.clip_end

        scene.camera = cam_obj
        space.lock_camera = True
        space.region_3d.view_perspective = 'CAMERA'

        return {'FINISHED'}


class CAMERA2VIEW_OT_KeyframeCamera(bpy.types.Operator):
    bl_idname = "camera2view.keyframe_camera"
    bl_label = "Keyframe Active Camera"
    bl_description = "Keyframe the active camera's location and rotation"

    def execute(self, context):
        cam = context.scene.camera
        if cam is None:
            self.report({'WARNING'}, "No active camera to keyframe")
            return {'CANCELLED'}

        cam.keyframe_insert(data_path="location")
        cam.keyframe_insert(data_path="rotation_euler")
        return {'FINISHED'}


class CAMERA2VIEW_OT_NextCamera(bpy.types.Operator):
    bl_idname = "camera2view.next_camera"
    bl_label = "Next Camera"
    bl_description = "Switch to the next Camera2View camera"

    def execute(self, context):
        cams = get_sorted_camera_objects()
        if not cams:
            self.report({'WARNING'}, "No Camera2View cameras found")
            return {'CANCELLED'}

        current = context.scene.camera
        try:
            index = cams.index(current)
            context.scene.camera = cams[(index + 1) % len(cams)]
        except ValueError:
            context.scene.camera = cams[0]

        return {'FINISHED'}


class CAMERA2VIEW_OT_PrevCamera(bpy.types.Operator):
    bl_idname = "camera2view.prev_camera"
    bl_label = "Previous Camera"
    bl_description = "Switch to the previous Camera2View camera"

    def execute(self, context):
        cams = get_sorted_camera_objects()
        if not cams:
            self.report({'WARNING'}, "No Camera2View cameras found")
            return {'CANCELLED'}

        current = context.scene.camera
        try:
            index = cams.index(current)
            context.scene.camera = cams[(index - 1) % len(cams)]
        except ValueError:
            context.scene.camera = cams[-1]

        return {'FINISHED'}


class CAMERA2VIEW_OT_NextKeyframe(bpy.types.Operator):
    bl_idname = "camera2view.next_keyframe"
    bl_label = "Next Keyframe"
    bl_description = "Move to next keyframe of active Camera2View"

    def execute(self, context):
        cam = context.scene.camera
        if not cam or not cam.name.startswith("Camera2View_"):
            self.report({'WARNING'}, "Active object is not a Camera2View camera")
            return {'CANCELLED'}

        keyframes = []
        if cam.animation_data and cam.animation_data.action:
            keyframes = sorted({kp.co.x for fcurve in cam.animation_data.action.fcurves for kp in fcurve.keyframe_points})
        current_frame = context.scene.frame_current
        next_keys = [f for f in keyframes if f > current_frame]

        if next_keys:
            context.scene.frame_current = int(next_keys[0])
        else:
            self.report({'INFO'}, "No next keyframe found")

        return {'FINISHED'}


class CAMERA2VIEW_OT_PrevKeyframe(bpy.types.Operator):
    bl_idname = "camera2view.prev_keyframe"
    bl_label = "Previous Keyframe"
    bl_description = "Move to previous keyframe of active Camera2View"

    def execute(self, context):
        cam = context.scene.camera
        if not cam or not cam.name.startswith("Camera2View_"):
            self.report({'WARNING'}, "Active object is not a Camera2View camera")
            return {'CANCELLED'}

        keyframes = []
        if cam.animation_data and cam.animation_data.action:
            keyframes = sorted({kp.co.x for fcurve in cam.animation_data.action.fcurves for kp in fcurve.keyframe_points})
        current_frame = context.scene.frame_current
        prev_keys = [f for f in keyframes if f < current_frame]

        if prev_keys:
            context.scene.frame_current = int(prev_keys[-1])
        else:
            self.report({'INFO'}, "No previous keyframe found")

        return {'FINISHED'}


class CAMERA2VIEW_OT_KeyframeProperty(bpy.types.Operator):
    bl_idname = "camera2view.keyframe_property"
    bl_label = "Keyframe This Property"
    bl_description = "Insert a keyframe for the specified camera property"

    property_name: bpy.props.StringProperty()

    def execute(self, context):
        cam = context.scene.camera
        if not cam or not cam.name.startswith("Camera2View_"):
            self.report({'WARNING'}, "Active object is not a Camera2View_* camera")
            return {'CANCELLED'}

        try:
            cam.data.keyframe_insert(data_path=self.property_name)
            return {'FINISHED'}
        except Exception as e:
            self.report({'ERROR'}, f"Failed to keyframe: {e}")
            return {'CANCELLED'}


classes = (
    CAMERA2VIEW_OT_CreateCamera,
    CAMERA2VIEW_OT_KeyframeCamera,
    CAMERA2VIEW_OT_NextCamera,
    CAMERA2VIEW_OT_PrevCamera,
    CAMERA2VIEW_OT_NextKeyframe,
    CAMERA2VIEW_OT_PrevKeyframe,
    CAMERA2VIEW_OT_KeyframeProperty,
)


def register():
    for cls in classes:
        bpy.utils.register_class(cls)


def unregister():
    for cls in reversed(classes):
        bpy.utils.unregister_class(cls)
